@@ -10,10 +10,12 @@
 #
 # Author: Michael Rash (mbr@cipherdyne.org)
 #
-# Version: 0.9.9
+# Version: 0.9
 #
 ##############################################################################
 #
+# $Id: ChainMgr.pm 990 2008-02-02 19:01:21Z mbr $
+#
 
 package IPTables::ChainMgr;
 
@@ -21,12 +23,12 @@ use 5.006;
 use POSIX ':sys_wait_h';
 use Carp;
 use IPTables::Parse;
-use NetAddr::IP;
+use Net::IPv4Addr 'ipv4_network';
 use strict;
 use warnings;
 use vars qw($VERSION);
 
-$VERSION = '0.9.9';
+$VERSION = '0.9';
 
 sub new() {
     my $class = shift;
@@ -47,17 +49,13 @@ sub new() {
         unless -e $self->{'_iptables'};
     croak "[*] $self->{'_iptables'} not executable.\n"
         unless -x $self->{'_iptables'};
-
-    $self->{'_ipt_bin_name'} = 'iptables';
-    $self->{'_ipt_bin_name'} = $1 if $self->{'_iptables'} =~ m|.*/(\S+)|;
-
     bless $self, $class;
 }
 
 sub chain_exists() {
     my $self = shift;
     my $table = shift || croak '[*] Must specify a table, e.g. "filter".';
-    my $chain = shift || croak '[*] Must specify a chain to check.';
+    my $chain = shift || croak '[*] Must specify a chain to create.';
     my $iptables = $self->{'_iptables'};
 
     ### see if the chain exists
@@ -112,14 +110,11 @@ sub delete_chain() {
     ### could not flush the chain
     return 0, $out_aref, $err_aref unless $rv;
 
-    my $ip_any_net = '0.0.0.0/0';
-    $ip_any_net = '::/0' if $self->{'_ipt_bin_name'} eq 'ip6tables';
-
     ### find and delete jump rules to this chain (we can't delete
     ### the chain until there are no references to it)
     my ($rulenum, $num_chain_rules)
-        = $self->find_ip_rule($ip_any_net, $ip_any_net,
-            $table, $jump_from_chain, $del_chain, {});
+        = $self->find_ip_rule('0.0.0.0/0',
+            '0.0.0.0/0', $table, $jump_from_chain, $del_chain, {});
 
     if ($rulenum) {
         $self->run_ipt_cmd(
@@ -157,10 +152,10 @@ sub append_ip_rule() {
 
     if ($rule_position) {
         my $msg = '';
-        if (keys %$extended_href) {
+        if ($extended_href) {
             $msg = "Table: $table, chain: $chain, $normalized_src -> " .
                 "$normalized_dst ";
-            for my $key (qw(protocol s_port d_port mac_source)) {
+            for my $key qw(protocol s_port d_port mac_source) {
                 $msg .= "$key $extended_href->{$key} "
                     if defined $extended_href->{$key};
             }
@@ -177,7 +172,7 @@ sub append_ip_rule() {
     my $msg     = '';
     my $idx_err = '';
 
-    if (keys %$extended_href) {
+    if ($extended_href) {
         $ipt_cmd = "$iptables -t $table -A $chain ";
         $ipt_cmd .= "-p $extended_href->{'protocol'} "
             if defined $extended_href->{'protocol'};
@@ -193,7 +188,7 @@ sub append_ip_rule() {
 
         $msg = "Table: $table, chain: $chain, added $normalized_src " .
             "-> $normalized_dst ";
-        for my $key (qw(protocol s_port d_port mac_source)) {
+        for my $key qw(protocol s_port d_port mac_source) {
             $msg .= "$key $extended_href->{$key} "
                 if defined $extended_href->{$key};
         }
@@ -246,10 +241,10 @@ sub add_ip_rule() {
 
     if ($rule_position) {
         my $msg = '';
-        if (keys %$extended_href) {
+        if ($extended_href) {
             $msg = "Table: $table, chain: $chain, $normalized_src -> " .
                 "$normalized_dst ";
-            for my $key (qw(protocol s_port d_port mac_source)) {
+            for my $key qw(protocol s_port d_port mac_source) {
                 $msg .= "$key $extended_href->{$key} "
                     if defined $extended_href->{$key};
             }
@@ -276,7 +271,7 @@ sub add_ip_rule() {
     }
     $rulenum = 1 if $rulenum == 0;
 
-    if (keys %$extended_href) {
+    if ($extended_href) {
         $ipt_cmd = "$iptables -t $table -I $chain $rulenum ";
         $ipt_cmd .= "-p $extended_href->{'protocol'} "
             if defined $extended_href->{'protocol'};
@@ -292,7 +287,7 @@ sub add_ip_rule() {
 
         $msg = "Table: $table, chain: $chain, added $normalized_src " .
             "-> $normalized_dst ";
-        for my $key (qw(protocol s_port d_port mac_source)) {
+        for my $key qw(protocol s_port d_port mac_source) {
             $msg .= "$key $extended_href->{$key} "
                 if defined $extended_href->{$key};
         }
@@ -348,8 +343,8 @@ sub delete_ip_rule() {
     }
 
     my $extended_msg = '';
-    if (keys %$extended_href) {
-        for my $key (qw(protocol s_port d_port mac_source)) {
+    if ($extended_href) {
+        for my $key qw(protocol s_port d_port mac_source) {
             $extended_msg .= "$key: $extended_href->{$key} "
                 if defined $extended_href->{$key};
         }
@@ -375,7 +370,6 @@ sub find_ip_rule() {
     my $chain = shift || croak '[*] Must specify iptables chain.';
     my $target = shift ||
         croak '[*] Must specify iptables target (this may be a chain).';
-
     ### optionally add port numbers and protocols, etc.
     my $extended_href = shift || {};
     my $iptables = $self->{'_iptables'};
@@ -402,25 +396,20 @@ sub find_ip_rule() {
 
     my $chain_aref = $ipt_parse->chain_rules($table, $chain);
 
-    $src = $self->normalize_net($src) if defined $extended_href->{'normalize'}
-        and $extended_href->{'normalize'};
-    $dst = $self->normalize_net($dst) if defined $extended_href->{'normalize'}
-        and $extended_href->{'normalize'};
-
     my $rulenum = 1;
     for my $rule_href (@$chain_aref) {
         if ($rule_href->{'target'} eq $target
                 and $rule_href->{'src'} eq $src
                 and $rule_href->{'dst'} eq $dst) {
-            if (keys %$extended_href) {
+            if ($extended_href) {
                 my $found = 1;
-                for my $key (qw(
+                for my $key qw(
                     protocol
                     s_port
                     d_port
                     to_ip
                     to_port
-                )) {
+                ) {
                     if (defined $extended_href->{$key}) {
                         unless ($extended_href->{$key}
                                 eq $rule_href->{$key}) {
@@ -453,21 +442,19 @@ sub normalize_net() {
     my $self = shift;
     my $net  = shift || croak '[*] Must specify net.';
 
-    my $normalized_net = $net;  ### establish default
+    ### regex to match an IP address
+    my $ip_re = '(?:\d{1,3}\.){3}\d{1,3}';
 
-    ### regex to match an IPv4 address
-    my $ipv4_re = qr/(?:\d{1,3}\.){3}\d{1,3}/;
-
-    if ($net =~ m|/| and $net =~ $ipv4_re or $net =~ m|:|) {
-        if ($net =~ m|:|) {  ### an IPv6 address
-            my $n = new6 NetAddr::IP $net
-                or croak "[*] Could not acquire NetAddr::IP object for $net";
-            $normalized_net = lc($n->network()->short()) . '/' . $n->masklen();
-        } else {
-            my $n = new NetAddr::IP $net
-                or croak "[*] Could not acquire NetAddr::IP object for $net";
-            $normalized_net = $n->network()->cidr();
-        }
+    my $normalized_net = '';
+    if ($net =~ m|($ip_re)/($ip_re)|) {
+        my ($net_addr, $cidr) = ipv4_network($1, $2);
+        $normalized_net = "$net_addr/$cidr";
+    } elsif ($net =~ m|($ip_re)/(\d+)|) {
+        my ($net_addr, $cidr) = ipv4_network($1, $2);
+        $normalized_net = "$net_addr/$cidr";
+    } else {
+        ### it is a hostname or an individual IP
+        $normalized_net = $net;
     }
     return $normalized_net;
 }
@@ -486,12 +473,9 @@ sub add_jump_rule() {
             "not allowed."], [];
     }
 
-    my $ip_any_net = '0.0.0.0/0';
-    $ip_any_net = '::/0' if $self->{'_ipt_bin_name'} eq 'ip6tables';
-
     ### first check to see if the jump rule already exists
     my ($rule_position, $num_chain_rules)
-        = $self->find_ip_rule($ip_any_net, $ip_any_net, $table,
+        = $self->find_ip_rule('0.0.0.0/0', '0.0.0.0/0', $table,
             $from_chain, $to_chain, {});
 
     ### check to see if the insertion index ($rulenum) is too big
@@ -539,10 +523,8 @@ sub run_ipt_cmd() {
     my $ipt_exec_sleep = $self->{'_ipt_exec_sleep'};
     my $sigchld_handler = $self->{'_sigchld_handler'};
 
-
     croak "[*] $cmd does not look like an iptables command."
-        unless $cmd =~ m|^\s*iptables| or $cmd =~ m|^\S+/iptables|
-            or $cmd =~ m|^\s*ip6tables| or $cmd =~ m|^\S+/ip6tables|;
+        unless $cmd =~ m|^\s*iptables| or $cmd =~ m|^\S+/iptables|;
 
     my $rv = 1;
     my @stdout = ();
@@ -651,16 +633,14 @@ __END__
 
 =head1 NAME
 
-IPTables::ChainMgr - Perl extension for manipulating iptables and ip6tables policies
+IPTables::ChainMgr - Perl extension for manipulating iptables policies
 
 =head1 SYNOPSIS
 
   use IPTables::ChainMgr;
 
-  my $ipt_bin = '/sbin/iptables'; # can set this to /sbin/ip6tables
-
   my %opts = (
-      'iptables' => $ipt_bin,
+      'iptables' => '/sbin/iptables',
       'iptout'   => '/tmp/iptables.out',
       'ipterr'   => '/tmp/iptables.err',
       'debug'    => 0,
@@ -697,88 +677,52 @@ IPTables::ChainMgr - Perl extension for manipulating iptables and ip6tables poli
   # create new iptables chain in the 'filter' table
   $ipt_obj->create_chain('filter', 'CUSTOM');
 
-  # translate a network into the same representation that iptables or
-  # ip6tables uses (e.g. '10.1.2.3/24' is properly represented as '10.1.2.0/24',
-  # and '0000:0000:00AA:0000:0000:AA00:0000:0001/64' = '0:0:aa::/64')
-  $normalized_net = $ipt_obj->normalize_net('10.1.2.3/24');
-
   # add rule to jump packets from the INPUT chain into CUSTOM at the
   # 4th rule position
   $ipt_obj->add_jump_rule('filter', 'INPUT', 4, 'CUSTOM');
 
-  # find rule that allows all traffic from 10.1.2.0/24 to 192.168.1.2
-  ($rv, $rule_num) = $ipt_obj->find_ip_rule('10.1.2.0/24', '192.168.1.2',
-      'filter', 'INPUT', 'ACCEPT', {'normalize' => 1});
+  # find rule that allows all traffic from 10.1.2.3 to 192.168.1.2
+  ($rv, $rule_num) = $ipt_obj->find_ip_rule('10.1.2.3', '192.168.1.2',
+      'filter', 'INPUT', 'ACCEPT', {});
 
-  # find rule that allows all TCP port 80 traffic from 10.1.2.0/24 to
+  # find rule that allows all TCP port 80 traffic from 10.1.2.3 to
   # 192.168.1.1
-  ($rv, $rule_num) = $ipt_obj->find_ip_rule('10.1.2.0/24', '192.168.1.2',
-      'filter', 'INPUT', 'ACCEPT', {'normalize' => 1, 'protocol' => 'tcp',
-      's_port' => 0, 'd_port' => 80});
+  ($rv, $rule_num) = $ipt_obj->find_ip_rule('10.1.2.3', '192.168.1.2',
+      'filter', 'INPUT', 'ACCEPT', {'protocol' => 'tcp', 's_port' => 0,
+      'd_port' => 80});
 
   # add rule at the 5th rule position to allow all traffic from
-  # 10.1.2.0/24 to 192.168.1.2 via the INPUT chain in the filter table
-  ($rv, $out_ar, $errs_ar) = $ipt_obj->add_ip_rule('10.1.2.0/24',
+  # 10.1.2.3 to 192.168.1.2 via the INPUT chain in the filter table
+  ($rv, $out_ar, $errs_ar) = $ipt_obj->add_ip_rule('10.1.2.3',
       '192.168.1.2', 5, 'filter', 'INPUT', 'ACCEPT', {});
 
   # add rule at the 4th rule position to allow all traffic from
-  # 10.1.2.0/24 to 192.168.1.2 over TCP port 80 via the CUSTOM chain
+  # 10.1.2.3 to 192.168.1.2 over TCP port 80 via the CUSTOM chain
   # in the filter table
-  ($rv, $out_ar, $errs_ar) = $ipt_obj->add_ip_rule('10.1.2.0/24',
+  ($rv, $out_ar, $errs_ar) = $ipt_obj->add_ip_rule('10.1.2.3',
       '192.168.1.2', 4, 'filter', 'CUSTOM', 'ACCEPT',
       {'protocol' => 'tcp', 's_port' => 0, 'd_port' => 80});
 
   # append rule at the end of the CUSTOM chain in the filter table to
-  # allow all traffic from 10.1.2.0/24 to 192.168.1.2 via port 80
-  ($rv, $out_ar, $errs_ar) = $ipt_obj->append_ip_rule('10.1.2.0/24',
+  # allow all traffic from 10.1.2.3 to 192.168.1.2 via port 80
+  ($rv, $out_ar, $errs_ar) = $ipt_obj->append_ip_rule('10.1.2.3',
       '192.168.1.2', 'filter', 'CUSTOM', 'ACCEPT',
       {'protocol' => 'tcp', 's_port' => 0, 'd_port' => 80});
 
-  # for each of the examples above, here are ip6tables analogs
-  # (requires instantiating the IPTables::ChainMgr object with
-  # /sbin/ip6tables): find rule that allows all traffic from fe80::200:f8ff:fe21:67cf
-  # to 0:0:aa::/64
-  ($rv, $rule_num) = $ipt_obj->find_ip_rule('fe80::200:f8ff:fe21:67cf', '0:0:aa::/64',
-      'filter', 'INPUT', 'ACCEPT', {'normalize' => 1});
-
-  # find rule that allows all TCP port 80 traffic from fe80::200:f8ff:fe21:67c to 0:0:aa::/64
-  ($rv, $rule_num) = $ipt_obj->find_ip_rule('fe80::200:f8ff:fe21:67cf', '0:0:aa::/64',
-      'filter', 'INPUT', 'ACCEPT', {'normalize' => 1, 'protocol' => 'tcp',
-      's_port' => 0, 'd_port' => 80});
-
-  # add rule at the 5th rule position to allow all traffic from
-  # fe80::200:f8ff:fe21:67c to 0:0:aa::/64 via the INPUT chain in the filter table
-  ($rv, $out_ar, $errs_ar) = $ipt_obj->add_ip_rule('fe80::200:f8ff:fe21:67cf',
-      '0:0:aa::/64', 5, 'filter', 'INPUT', 'ACCEPT', {});
-
-  # add rule at the 4th rule position to allow all traffic from
-  # fe80::200:f8ff:fe21:67c to 0:0:aa::/64 over TCP port 80 via the CUSTOM chain
-  # in the filter table
-  ($rv, $out_ar, $errs_ar) = $ipt_obj->add_ip_rule('fe80::200:f8ff:fe21:67cf',
-      '0:0:aa::/64', 4, 'filter', 'CUSTOM', 'ACCEPT',
-      {'protocol' => 'tcp', 's_port' => 0, 'd_port' => 80});
-
-  # append rule at the end of the CUSTOM chain in the filter table to
-  # allow all traffic from fe80::200:f8ff:fe21:67c to 0:0:aa::/64 via port 80
-  ($rv, $out_ar, $errs_ar) = $ipt_obj->append_ip_rule('fe80::200:f8ff:fe21:67cf',
-      '0:0:aa::/64', 'filter', 'CUSTOM', 'ACCEPT',
-      {'protocol' => 'tcp', 's_port' => 0, 'd_port' => 80});
-
   # run an arbitrary iptables command and collect the output
   ($rv, $out_ar, $errs_ar) = $ipt_obj->run_ipt_cmd(
           '/sbin/iptables -v -n -L');
 
 =head1 DESCRIPTION
 
-The C<IPTables::ChainMgr> package provides an interface to manipulate iptables
-and ip6tables policies on Linux systems through the direct execution of
-iptables/ip6tables commands.  Although making a perl extension of libiptc
-provided by the iptables project is possible (and has been done by the
-IPTables::libiptc module available from CPAN), it is also easy enough to just
-execute iptables/ip6tables commands directly in order to both parse and change
-the configuration of the policy.  Further, this simplifies installation since
-the only external requirement is (in the spirit of scripting) to be able to
-point IPTables::ChainMgr at an installed iptables or ip6tables binary instead
+The C<IPTables::ChainMgr> package provide an interface to manipulate iptables
+policies on Linux systems through the direct execution of iptables commands.
+Although making a perl extension of libiptc provided by the iptables project is
+possible (and has been done by the IPTables::libiptc module available from CPAN),
+it is also easy enough to just execute iptables commands directly in order to
+both parse and change the configuration of the policy.  Further, this simplifies
+installation since the only external requirement is (in the spirit of scripting)
+to be able to point IPTables::ChainMgr at an installed iptables binary instead
 of having to compile against a library.
 
 =head1 FUNCTIONS
@@ -792,21 +736,18 @@ functions:
 
 This function tests whether or not a chain (e.g. 'INPUT') exists within the
 specified table (e.g. 'filter').  This is most useful to test whether
-a custom chain has been added to the running iptables/ip6tables policy.  The
-return values are (as with many IPTables::ChainMgr functions) an array of
-three things: a numeric value, and both the stdout and stderr of the iptables
-or ip6tables command in the form of array references.  So, an example
-invocation of the chain_exists() function would be:
+a custom chain has been added to the running iptables policy.  The return values
+are (as with many IPTables::ChainMgr functions) an array of three things: a
+numeric value, and both the stdout and stderr of the iptables command in the
+form of array references.  So, an example invocation of the chain_exists()
+function would be:
 
   ($rv, $out_ar, $errs_ar) = $ipt_obj->chain_exists('filter', 'CUSTOM');
 
 If $rv is 1, then the CUSTOM chain exists in the filter table, and 0 otherwise.
 The $out_ar array reference contains the output of the command "/sbin/iptables -t filter -v -n -L CUSTOM",
 which will contain the rules in the CUSTOM chain (if it exists) or nothing (if not).
-The $errs_ar array reference contains the stderr of the iptables command.  As
-with all IPTables::ChainMgr functions, if the IPTables::ChainMgr object was
-instantiated with the ip6tables binary path, then the above command would
-become "/sbin/ip6tables -t filter -v -n -L CUSTOM".
+The $errs_ar array reference contains the stderr of the iptables command.
 
 =item create_chain($table, $chain)
 
@@ -816,8 +757,7 @@ values are given like so:
   ($rv, $out_ar, $errs_ar) = $ipt_obj->create_chain('filter', 'CUSTOM');
 
 Behind the scenes, the create_chain() function in the example above runs the
-iptables command "/sbin/iptables -t filter -N CUSTOM", or for ip6tables
-"/sbin/ip6tables -t filter -N CUSTOM".
+iptables command "/sbin/iptables -t filter -N CUSTOM".
 
 =item flush_chain($table, $chain)
 
@@ -827,7 +767,7 @@ values are returned:
   ($rv, $out_ar, $errs_ar) = $ipt_obj->flush_chain('filter', 'CUSTOM');
 
 The flush_chain() function in the example above executes the iptables command
-"/sbin/iptables -t filter -F CUSTOM" or "/sbin/ip6tables -t filter -F CUSTOM".
+"/sbin/iptables -t filter -F CUSTOM"
 
 =item delete_chain($table, $jump_from_chain, $chain)
 
@@ -848,34 +788,19 @@ This function parses the specified chain to see if there is a rule that
 matches the $src, $dst, $target, and (optionally) any %extended_info
 criteria.  The return values are the rule number in the chain (or zero
 if it doesn't exist), and the total number of rules in the chain.  Below
-are four examples; the first is to find an ACCEPT rule for 10.1.2.0/24 to
+are two examples; the first is to find an ACCEPT rule for 10.1.2.3 to
 communicate with 192.168.1.2 in the INPUT chain, and the second is the
-same except that the rule is restricted to TCP port 80.  The third and
-forth examples illustrate ip6tables analogs of the first two examples
-with source IP fe80::200:f8ff:fe21:67cf/128 and destination network: 0:0:aa::/64
+same except that the rule is restricted to TCP port 80:
 
-  ($rulenum, $chain_rules) = $ipt_obj->find_ip_rule('10.1.2.0/24',
-      '192.168.1.2', 'filter', 'INPUT', 'ACCEPT', {'normalize' => 1});
+  ($rulenum, $chain_rules) = $ipt_obj->find_ip_rule('10.1.2.3',
+      '192.168.1.2', 'filter', 'INPUT', 'ACCEPT', {});
   if ($rulenum) {
       print "matched rule $rulenum out of $chain_rules rules\n";
   }
 
-  ($rulenum, $chain_rules) = $ipt_obj->find_ip_rule('10.1.2.0/24',
+  ($rulenum, $chain_rules) = $ipt_obj->find_ip_rule('10.1.2.3',
       '192.168.1.2', 'filter', 'INPUT', 'ACCEPT',
-      {'normalize' => 1, 'protocol' => 'tcp', 's_port' => 0, 'd_port' => 80});
-  if ($rulenum) {
-      print "matched rule $rulenum out of $chain_rules rules\n";
-  }
-
-  ($rulenum, $chain_rules) = $ipt_obj->find_ip_rule('fe80::200:f8ff:fe21:67cf/128',
-    '0:0:aa::/64', 'filter', 'INPUT', 'ACCEPT', {'normalize' => 1});
-  if ($rulenum) {
-      print "matched rule $rulenum out of $chain_rules rules\n";
-  }
-
-  ($rulenum, $chain_rules) = $ipt_obj->find_ip_rule('fe80::200:f8ff:fe21:67cf/128',
-      '0:0:aa::/64', 'filter', 'INPUT', 'ACCEPT',
-      {'normalize' => 1, 'protocol' => 'tcp', 's_port' => 0, 'd_port' => 80});
+      {'protocol' => 'tcp', 's_port' => 0, 'd_port' => 80});
   if ($rulenum) {
       print "matched rule $rulenum out of $chain_rules rules\n";
   }
@@ -908,15 +833,6 @@ jumped to the CUSTOM chain from the INPUT chain at rule 4:
 
   ($rv, $out_ar, $errs_ar) = $ipt_obj->add_jump_rule('filter', 'INPUT', 4, 'CUSTOM');
 
-=item normalize_net($net)
-
-This function translates an IP/network into the same representation that iptables
-or ip6tables uses upon listing a policy.  The first example shows an IPv4 network
-and how iptables lists it, and the second is an IPv6 network:
-
-  print $ipt_obj->normalize_net('10.1.2.3/24'), "\n" # prints '10.1.2.0/24'
-  print $ipt_obj->normalize_net('0000:0000:00AA:0000:0000:AA00:0000:0001/64'), "\n" # prints '0:0:aa::/64'
-
 =item run_ipt_cmd($cmd)
 
 This function is a generic work horse function for executing iptables commands,
@@ -940,16 +856,18 @@ and stderr.  Here is an example to list all rules in the user-defined chain
 =head1 SEE ALSO
 
 The IPTables::ChainMgr extension is closely associated with the IPTables::Parse
-extension, and both are heavily used by the psad and fwsnort projects to
-manipulate iptables policies based on various criteria (see the psad(8) and
-fwsnort(8) man pages).  As always, the iptables(8) man page provides the best
-information on command line execution and theory behind iptables.
+extension, and both are heavily used by the psad, fwsnort, and fwknop projects
+to manipulate iptables policies based on various criteria (see the psad(8),
+fwsnort(8), and fwknop(8) man pages).  As always, the iptables(8) man page
+provides the best information on command line execution and theory behind
+iptables.
 
 Although there is no mailing that is devoted specifically to the IPTables::ChainMgr
 extension, questions about the extension will be answered on the following
 lists:
 
   The psad mailing list: http://lists.sourceforge.net/lists/listinfo/psad-discuss
+  The fwknop mailing list: http://lists.sourceforge.net/lists/listinfo/fwknop-discuss
   The fwsnort mailing list: http://lists.sourceforge.net/lists/listinfo/fwsnort-discuss
 
 The latest version of the IPTables::ChainMgr extension can be found at:
@@ -967,12 +885,12 @@ Thanks to the following people:
 =head1 AUTHOR
 
 The IPTables::ChainMgr extension was written by Michael Rash F<E<lt>mbr@cipherdyne.orgE<gt>>
-to support the psad and fwsnort projects.  Please send email to this address if
-there are any questions, comments, or bug reports.
+to support the psad, fwknop, and fwsnort projects.  Please send email to
+this address if there are any questions, comments, or bug reports.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005-2012 by Michael Rash
+Copyright (C) 2005-2008 by Michael Rash
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.5 or,
